/***************************************************************
 * Name:      listctrl1.cpp
 * Purpose:   Code for Dialog listctrl1
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-12-02
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "listctrl1.h"

wxBEGIN_EVENT_TABLE(listctrl1, wxDialog)
    EVT_CLOSE(listctrl1::OnClose)
    EVT_BUTTON(wxID_OK, listctrl1::OnOk)
    EVT_LIST_ITEM_SELECTED(idListCtrl, listctrl1::OnItemSelected)
    EVT_LIST_ITEM_ACTIVATED(idListCtrl, listctrl1::OnItemactivated)
    EVT_LIST_ITEM_FOCUSED(idListCtrl, listctrl1::OnItemFocused)
    EVT_LIST_INSERT_ITEM(idListCtrl, listctrl1::OnInsertItem)
    EVT_RADIOBOX(idRadioBox, listctrl1::OnRadioButton)
wxEND_EVENT_TABLE()

listctrl1::listctrl1(wxWindow *wnd, datosRepuestos &datos)
    : wxDialog(wnd, -1, _T("wxListCtrl selección de ítems")), data(datos)
{
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    listCtrl = new wxListCtrl(this, idListCtrl, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
    boxVertical->Add(listCtrl, wxSizerFlags().Expand().Border());

    wxArrayString Opciones;
    Opciones.Add(_T("&Lista"));
    Opciones.Add(_T("&Reporte"));
    Opciones.Add(_T("Iconos &grandes"));
    Opciones.Add(_T("Iconos &pequeños"));

    radioBox = new wxRadioBox(this, idRadioBox, _T("Modo"), wxDefaultPosition, wxDefaultSize,
        Opciones, 4, wxRA_SPECIFY_ROWS);
    boxVertical->Add(radioBox, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    radioBox->SetSelection(1); // Seleccionar modo reporte
    AsignaImagenes(listCtrl);
    InsertaItems(listCtrl, reporte);
}

listctrl1::~listctrl1()
{
    //dtor
}

void listctrl1::OnClose(wxCloseEvent& event) {
    Destroy();
}

void listctrl1::InsertaColumnas(wxListCtrl* listCtrl) {
    listCtrl->InsertColumn(0, _T("Nombre"));
    wxListItem listitem;
    listitem.SetAlign(wxLIST_FORMAT_RIGHT);
    listitem.SetText(_T("Stock"));
    listCtrl->InsertColumn(1, listitem);
    listitem.SetText(_T("Consumido"));
    listCtrl->InsertColumn(2, listitem);
}

void listctrl1::AsignaImagenes(wxListCtrl* listCtrl) {
    wxBitmap bm;
    wxImageList* listaImagenGrande=new wxImageList(48, 48, true, 3);
    bm.LoadFile("tornillo-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    bm.LoadFile("tuerca-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    listCtrl->AssignImageList(listaImagenGrande, wxIMAGE_LIST_NORMAL);
    wxImageList* listaImagenPequena=new wxImageList(16, 16, true, 3);
    bm.LoadFile("tornillo-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    bm.LoadFile("tuerca-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    listCtrl->AssignImageList(listaImagenPequena, wxIMAGE_LIST_SMALL);
}

void listctrl1::InsertaItems(wxListCtrl* listCtrl, lc_modo modo) {
    wxString cad;
    wxListItem item;

    if(modo == reporte) InsertaColumnas(listCtrl);
    for(size_t i=0; i<data.almacen.size(); i++) {
        item.Clear();
        item.SetId(i);
        item.SetColumn(0);
        item.SetImage(data.almacen[i].tipo);
        item.SetText(data.almacen[i].nombre);
        item.SetData(data.almacen[i].clave);
        listCtrl->InsertItem(item);
        if(modo == reporte) {
            cad.Clear();
            cad << data.almacen[i].stock;
            listCtrl->SetItem(i, 1, cad);
            cad.Clear();
            cad << data.almacen[i].consumido;
            listCtrl->SetItem(i, 2, cad);
        }
    }
    item.Clear();
    for(size_t i=0; i<data.marcado.size(); i++) {
        item.SetId(data.marcado[i]);
//        item.SetMask(wxLIST_MASK_STATE);
//        item.SetStateMask(wxLIST_STATE_SELECTED);
        item.SetState(wxLIST_STATE_SELECTED);
        listCtrl->SetItem(item);
    }
}

void listctrl1::OnItemSelected(wxListEvent& event) {
    wxString cad;
    if(event.GetId()!=idListCtrl) {
        cad.Printf(_T("Item seleccionado: #%ld, dato: %llu"), event.GetIndex(), reinterpret_cast<size_t>(event.GetData()));
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl1::OnInsertItem(wxListEvent& event) {
    wxString cad;
    if(event.GetId()!=idListCtrl) {
        cad.Printf(_T("Item insertado: #%ld, dato: %llu"), event.GetIndex(), reinterpret_cast<size_t>(event.GetData()));
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl1::OnItemactivated(wxListEvent& event) {
    wxString cad;
    if(event.GetId()==idListCtrl) {
        cad.Printf(_T("Item activado: #%ld"), event.GetIndex());
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl1::OnItemFocused(wxListEvent& event) {
    wxString cad;
    if(event.GetId()==idListCtrl) {
        cad.Printf(_T("Item enfocado: #%ld"), event.GetIndex());
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl1::OnRadioButton(wxCommandEvent& event) {
    lc_modo modo;

    listCtrl->ClearAll();
    switch(event.GetSelection()) {
    case 0: // Lista
        listCtrl->SetWindowStyleFlag(wxLC_LIST);
        modo = lista;
        break;
    case 1: // Reporte
        listCtrl->SetWindowStyleFlag(wxLC_REPORT);
        modo = reporte;
        break;
    case 2: // Iconos grandes
        listCtrl->SetWindowStyleFlag(wxLC_ICON);
        modo = iconos;
        break;
    case 3: // Iconos pequeños
        listCtrl->SetWindowStyleFlag(wxLC_SMALL_ICON);
        modo = iconos_peq;
        break;
    }
    InsertaItems(listCtrl, modo);
    //void wxListCtrl::RefreshItems(long itemFrom, long itemTo);

    listCtrl->Arrange();
}

void listctrl1::OnOk(wxCommandEvent& event) {
    long item=-1;

    data.marcado.clear();
    do {
        item = listCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if(item!=-1) data.marcado.push_back(item);
    } while(item!=-1);
    EndModal(wxID_OK);
}
