/***************************************************************
 * Name:      listctrl2.cpp
 * Purpose:   Code for Dialog listctrl2
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-12-02
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "listctrl2.h"

wxBEGIN_EVENT_TABLE(listctrl2, wxDialog)
    EVT_CLOSE(listctrl2::OnClose)
    EVT_BUTTON(wxID_OK, listctrl2::OnOk)
    EVT_LIST_ITEM_SELECTED(idListCtrl, listctrl2::OnItemSelected)
    EVT_LIST_ITEM_ACTIVATED(idListCtrl, listctrl2::OnItemactivated)
    EVT_LIST_ITEM_FOCUSED(idListCtrl, listctrl2::OnItemFocused)
    EVT_LIST_INSERT_ITEM(idListCtrl, listctrl2::OnInsertItem)
    EVT_RADIOBOX(idRadioBox, listctrl2::OnRadioButton)
    EVT_CHECKLISTBOX(idCheckList, listctrl2::OnChecklist)
wxEND_EVENT_TABLE()

listctrl2::listctrl2(wxWindow *wnd, datosRepuestos &datos)
    : wxDialog(wnd, -1, _T("Marcas de checkeo")), data(datos)
{
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    listCtrl = new wxListCtrl(this, idListCtrl, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
    boxVertical->Add(listCtrl, wxSizerFlags().Expand().Border());

    wxArrayString Opciones;
    Opciones.Add(_T("&Lista"));
    Opciones.Add(_T("&Reporte"));
    Opciones.Add(_T("Iconos &grandes"));
    Opciones.Add(_T("Iconos &pequeños"));

    radioBox = new wxRadioBox(this, idRadioBox, _T("Modo"), wxDefaultPosition, wxDefaultSize,
        Opciones, 4, wxRA_SPECIFY_ROWS);
    boxVertical->Add(radioBox, wxSizerFlags().Expand().Border());

    wxString cads[] = {
        _T("Nombre"),
        _T("Stock"),
        _T("Consumido")
    };

    checkListBox = new wxCheckListBox(this, idCheckList, wxDefaultPosition, wxSize(200, wxDefaultCoord),
        sizeof(cads)/sizeof(cads[0]), cads);
    boxVertical->Add(checkListBox, wxSizerFlags().Expand().Border());
    for(int i=0; i<3; i++) checkListBox->Check(i);

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    listCtrl->EnableCheckBoxes();
    radioBox->SetSelection(1); // Seleccionar modo reporte
    AsignaImagenes(listCtrl);
    InsertaItems(listCtrl, reporte);
}

listctrl2::~listctrl2()
{
    //dtor
}

void listctrl2::OnClose(wxCloseEvent& event) {
    Destroy();
}

void listctrl2::InsertaColumnas(wxListCtrl* listCtrl) {
    listCtrl->InsertColumn(0, _T("Nombre"));
    wxListItem listitem;
    listitem.SetAlign(wxLIST_FORMAT_RIGHT);
    listitem.SetText(_T("Stock"));
    listCtrl->InsertColumn(1, listitem);
    listitem.SetText(_T("Consumido"));
    listCtrl->InsertColumn(2, listitem);
}

void listctrl2::AsignaImagenes(wxListCtrl* listCtrl) {
    wxBitmap bm;
    wxImageList* listaImagenGrande=new wxImageList(48, 48, true, 3);
    bm.LoadFile("tornillo-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    bm.LoadFile("tuerca-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    listCtrl->AssignImageList(listaImagenGrande, wxIMAGE_LIST_NORMAL);
    wxImageList* listaImagenPequena=new wxImageList(16, 16, true, 3);
    bm.LoadFile("tornillo-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    bm.LoadFile("tuerca-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    listCtrl->AssignImageList(listaImagenPequena, wxIMAGE_LIST_SMALL);
}

void listctrl2::InsertaItems(wxListCtrl* listCtrl, lc_modo modo) {
    wxString cad;
    wxListItem item;

    if(modo == reporte) InsertaColumnas(listCtrl);
    for(size_t i=0; i<data.almacen.size(); i++) {
        item.Clear();
        item.SetId(i);
        item.SetColumn(0);
        item.SetImage(data.almacen[i].tipo);
        item.SetText(data.almacen[i].nombre);
        item.SetData(data.almacen[i].clave);
        listCtrl->InsertItem(item);
        if(modo == reporte) {
            cad.Clear();
            cad << data.almacen[i].stock;
            listCtrl->SetItem(i, 1, cad);
            cad.Clear();
            cad << data.almacen[i].consumido;
            listCtrl->SetItem(i, 2, cad);
        }
    }
    item.Clear();
    for(size_t i=0; i<data.marcado.size(); i++) {
        listCtrl->CheckItem(data.marcado[i], true);
    }
    listCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
}

void listctrl2::OnItemSelected(wxListEvent& event) {
    wxString cad;
    if(event.GetId()!=idListCtrl) {
        cad.Printf(_T("Item seleccionado: #%ld, dato: %llu"), event.GetIndex(), reinterpret_cast<size_t>(event.GetData()));
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl2::OnInsertItem(wxListEvent& event) {
    wxString cad;
    if(event.GetId()!=idListCtrl) {
        cad.Printf(_T("Item insertado: #%ld, dato: %llu"), event.GetIndex(), reinterpret_cast<size_t>(event.GetData()));
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl2::OnItemactivated(wxListEvent& event) {
    wxString cad;
    if(event.GetId()==idListCtrl) {
        cad.Printf(_T("Item activado: #%ld"), event.GetIndex());
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl2::OnItemFocused(wxListEvent& event) {
    wxString cad;
    if(event.GetId()==idListCtrl) {
        cad.Printf(_T("Item enfocado: #%ld"), event.GetIndex());
        wxMessageBox(cad, _T("ListCtrl"));
    }
}

void listctrl2::OnRadioButton(wxCommandEvent& event) {
    lc_modo modo;

    listCtrl->ClearAll();
    switch(event.GetSelection()) {
    case 0: // Lista
        listCtrl->SetWindowStyleFlag(wxLC_LIST);
        modo = lista;
        break;
    case 1: // Reporte
        listCtrl->SetWindowStyleFlag(wxLC_REPORT);
        modo = reporte;
        break;
    case 2: // Iconos grandes
        listCtrl->SetWindowStyleFlag(wxLC_ICON);
        modo = iconos;
        break;
    case 3: // Iconos pequeños
        listCtrl->SetWindowStyleFlag(wxLC_SMALL_ICON);
        modo = iconos_peq;
        break;
    }
    InsertaItems(listCtrl, modo);
    checkListBox->Enable(event.GetSelection()==1);
    listCtrl->Arrange();
}

void listctrl2::OnChecklist(wxCommandEvent& event) {
    int indice=event.GetInt();
    bool estado = checkListBox->IsChecked(indice);
    if(estado) {
        listCtrl->SetColumnWidth(indice, ancho[indice]);
    } else {
        ancho[indice] = listCtrl->GetColumnWidth(indice);
        listCtrl->SetColumnWidth(indice, 0);
    }
}

void listctrl2::OnOk(wxCommandEvent& event) {
    wxListItem item;
    data.marcado.clear();
    for(int i=0; i<listCtrl->GetItemCount(); i++) {
        if(listCtrl->IsItemChecked(i)) data.marcado.push_back(i);
    }
    EndModal(wxID_OK);
}
