/***************************************************************
 * Name:      listctrl3.cpp
 * Purpose:   Code for Dialog listctrl3
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-12-02
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "listctrl3.h"

wxBEGIN_EVENT_TABLE(listctrl3, wxDialog)
    EVT_CLOSE(listctrl3::OnClose)
    EVT_RADIOBOX(idRadioBox, listctrl3::OnRadioButton)
    EVT_LIST_COL_CLICK(idListCtrl, listctrl3::OnListColClick)
    EVT_BUTTON(wxID_OK, listctrl3::OnOk)
wxEND_EVENT_TABLE()

listctrl3::listctrl3(wxWindow *wnd, datosRepuestos &datos)
    : wxDialog(wnd, -1, _T("Ordenar ítems")), data(datos)
{
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    listCtrl = new wxListCtrl(this, idListCtrl, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
    boxVertical->Add(listCtrl, wxSizerFlags().Expand().Border());

    wxArrayString Opciones;
    Opciones.Add(_T("&Lista"));
    Opciones.Add(_T("&Reporte"));
    Opciones.Add(_T("Iconos &grandes"));
    Opciones.Add(_T("Iconos &pequeños"));

    radioBox = new wxRadioBox(this, idRadioBox, _T("Modo"), wxDefaultPosition, wxDefaultSize,
        Opciones, 4, wxRA_SPECIFY_ROWS);
    boxVertical->Add(radioBox, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    radioBox->SetSelection(1); // Seleccionar modo reporte
    AsignaImagenes(listCtrl);
    InsertaItems(listCtrl, reporte);
    datos.ordenNombre=1;
    datos.ordenStock=1;
    datos.ordenConsumido=1;

}

listctrl3::~listctrl3()
{
    //dtor
}

void listctrl3::OnClose(wxCloseEvent& event) {
    Destroy();
}

void listctrl3::InsertaColumnas(wxListCtrl* listCtrl) {
    listCtrl->InsertColumn(0, _T("Nombre"));
    wxListItem listitem;
    listitem.SetAlign(wxLIST_FORMAT_RIGHT);
    listitem.SetText(_T("Stock"));
    listCtrl->InsertColumn(1, listitem);
    listitem.SetText(_T("Consumido"));
    listCtrl->InsertColumn(2, listitem);
}

void listctrl3::AsignaImagenes(wxListCtrl* listCtrl) {
    wxBitmap bm;
    wxImageList* listaImagenGrande=new wxImageList(48, 48, true, 3);
    bm.LoadFile("tornillo-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    bm.LoadFile("tuerca-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    listCtrl->AssignImageList(listaImagenGrande, wxIMAGE_LIST_NORMAL);
    wxImageList* listaImagenPequena=new wxImageList(16, 16, true, 3);
    bm.LoadFile("tornillo-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    bm.LoadFile("tuerca-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    listCtrl->AssignImageList(listaImagenPequena, wxIMAGE_LIST_SMALL);
}

void listctrl3::InsertaItems(wxListCtrl* listCtrl, lc_modo modo) {
    wxString cad;
    wxListItem item;

    if(modo == reporte) InsertaColumnas(listCtrl);
    for(size_t i=0; i<data.almacen.size(); i++) {
        item.Clear();
        item.SetId(i);
        item.SetColumn(0);
        item.SetImage(data.almacen[i].tipo);
        item.SetText(data.almacen[i].nombre);
        item.SetData(i);
        listCtrl->InsertItem(item);
        if(modo == reporte) {
            cad.Clear();
            cad << data.almacen[i].stock;
            listCtrl->SetItem(i, 1, cad);
            cad.Clear();
            cad << data.almacen[i].consumido;
            listCtrl->SetItem(i, 2, cad);
        }
    }
    for(size_t i=0; i<data.marcado.size(); i++) {
        item.SetId(data.marcado[i]);
        item.SetState(wxLIST_STATE_SELECTED);
        listCtrl->SetItem(item);
    }
    listCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
}

void listctrl3::OnRadioButton(wxCommandEvent& event) {
    lc_modo modo;

    listCtrl->ClearAll();
    switch(event.GetSelection()) {
    case 0: // Lista
        listCtrl->SetWindowStyleFlag(wxLC_LIST);
        modo = lista;
        break;
    case 1: // Reporte
        listCtrl->SetWindowStyleFlag(wxLC_REPORT);
        modo = reporte;
        break;
    case 2: // Iconos grandes
        listCtrl->SetWindowStyleFlag(wxLC_ICON);
        modo = iconos;
        break;
    case 3: // Iconos pequeños
        listCtrl->SetWindowStyleFlag(wxLC_SMALL_ICON);
        modo = iconos_peq;
        break;
    }
    InsertaItems(listCtrl, modo);
    listCtrl->Arrange();
}

void listctrl3::OnListColClick(wxListEvent& event) {
    switch(event.GetColumn()) {
    case 0:
        listCtrl->SortItems(ComparaNombres, reinterpret_cast<wxIntPtr>(&data));
        listCtrl->ShowSortIndicator(0, data.ordenNombre>0);
        data.ordenNombre *= -1;
        break;
    case 1:
        listCtrl->SortItems(ComparaStocks, reinterpret_cast<wxIntPtr>(&data));
        listCtrl->ShowSortIndicator(1, data.ordenStock>0);
        data.ordenStock *= -1;
        break;
    case 2:
        listCtrl->SortItems(ComparaConsumidos, reinterpret_cast<wxIntPtr>(&data));
        listCtrl->ShowSortIndicator(2, data.ordenConsumido>0);
        data.ordenConsumido *= -1;
        break;
    }
}

int wxCALLBACK ComparaNombres(wxIntPtr item1, wxIntPtr item2, wxIntPtr sortData) {
    wxString n1 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item1].nombre;
    wxString n2 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item2].nombre;
    int orden = reinterpret_cast<datosRepuestos*>(sortData)->ordenNombre;

    return n1.CmpNoCase(n2)*orden;
}

int wxCALLBACK ComparaStocks(wxIntPtr item1, wxIntPtr item2, wxIntPtr sortData) {
    int n1 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item1].stock;
    int n2 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item2].stock;
    int orden = reinterpret_cast<datosRepuestos*>(sortData)->ordenStock;

    if(n1>n2) return 1*orden;
    if(n1<n2) return -1*orden;
    return 0;
}

int wxCALLBACK ComparaConsumidos(wxIntPtr item1, wxIntPtr item2, wxIntPtr sortData) {
    int n1 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item1].consumido;
    int n2 = reinterpret_cast<datosRepuestos*>(sortData)->almacen[item2].consumido;
    int orden = reinterpret_cast<datosRepuestos*>(sortData)->ordenConsumido;

    if(n1>n2) return 1*orden;
    if(n1<n2) return -1*orden;
    return 0;
}

void listctrl3::OnOk(wxCommandEvent& event) {
    long item=-1;

    data.marcado.clear();
    do {
        item = listCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if(item!=-1) data.marcado.push_back(item);
    } while(item!=-1);
    EndModal(wxID_OK);
}
