/***************************************************************
 * Name:      listctrl5.cpp
 * Purpose:   Code for Dialog listctrl5
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-12-02
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/
#include "listctrl5.h"

miListCtrl::miListCtrl(wxWindow * parent, wxWindowID id) :
    wxListCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_VIRTUAL) {
    wxString consulta;
    sqlite3_stmt *ppStmt;
    int cuenta;

    estado = true;
    if(sqlite3_open("ciudades.db", &db) != SQLITE_OK) estado=false;
    else {
        consulta = _T("SELECT COUNT(*) FROM city;");
        sqlite3_prepare_v2(db, static_cast<const char*>(consulta), -1, &ppStmt, NULL);
        if(SQLITE_ROW == sqlite3_step(ppStmt)) {
            cuenta = (sqlite3_column_int(ppStmt, 0));
            sqlite3_finalize(ppStmt);
        }
        SetItemCount(cuenta);
    }
}

miListCtrl::~miListCtrl() {
    sqlite3_close(db);
}

wxString miListCtrl::OnGetItemText(long item, long column) const {
    wxString cad = _T("");
    wxString consulta;
    sqlite3_stmt *ppStmt;

    if(estado) {
        switch(column) {
        case 0:
            consulta.Printf(_T("SELECT Name FROM city WHERE ID='%d';"), item+1);
            break;
        case 1:
            consulta.Printf(_T("SELECT CountryCode FROM city WHERE ID='%d';"), item+1);
            break;
        case 2:
            consulta.Printf(_T("SELECT District FROM city WHERE ID='%d';"), item+1);
            break;
        case 3:
            consulta.Printf(_T("SELECT Population FROM city WHERE ID='%d';"), item+1);
            break;
        }
        sqlite3_prepare_v2(db, static_cast<const char*>(consulta), -1, &ppStmt, NULL);
        if(SQLITE_ROW == sqlite3_step(ppStmt)) {
            const char *p = reinterpret_cast<const char*>(sqlite3_column_text(ppStmt, 0));
            cad = wxString::FromUTF8(p);
            sqlite3_finalize(ppStmt);
        }
    }
    return cad;
}

int miListCtrl::OnGetItemImage(long item) const {
    return -1;
}

wxBEGIN_EVENT_TABLE(listctrl5, wxDialog)
    EVT_CLOSE(listctrl5::OnClose)
    EVT_BUTTON(wxID_OK, listctrl5::OnOk)
wxEND_EVENT_TABLE()

listctrl5::listctrl5(wxWindow *wnd, datosCiudades &datos)
    : wxDialog(wnd, -1, _T("wxListCtrl virtual")), data(datos)
{
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    listCtrl = new miListCtrl(this, idListCtrl);
    boxVertical->Add(listCtrl, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    InsertaColumnas(listCtrl);
    listCtrl->EnableAlternateRowColours();
}

listctrl5::~listctrl5()
{
    //dtor
}

void listctrl5::OnClose(wxCloseEvent& event) {
    Destroy();
}

void listctrl5::InsertaColumnas(wxListCtrl* listCtrl) {
    listCtrl->InsertColumn(0, _T("Nombre"));
    wxListItem listitem;
    //listitem.SetAlign(wxLIST_FORMAT_RIGHT);
    listitem.SetText(_T("País"));
    listCtrl->InsertColumn(1, listitem);
    listitem.SetText(_T("Zona"));
    listCtrl->InsertColumn(2, listitem);
    listitem.SetText(_T("Población"));
    listCtrl->InsertColumn(3, listitem);
}

void listctrl5::OnOk(wxCommandEvent& event) {
    long item=-1;
    wxListItem listitem;

    data.marcado.clear();
    do {
        item = listCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        listitem.SetMask(wxLIST_MASK_DATA);
        listitem.SetId(item);
        if(item!=-1) data.marcado.push_back(item);
    } while(item!=-1);
    EndModal(wxID_OK);
}
