/***************************************************************
 * Name:      listview.cpp
 * Purpose:   Code for Dialog listview
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-12-02
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/
#include "listview.h"

wxBEGIN_EVENT_TABLE(listview, wxDialog)
    EVT_CLOSE(listview::OnClose)
    EVT_BUTTON(wxID_OK, listview::OnOk)
wxEND_EVENT_TABLE()

listview::listview(wxWindow *wnd, datosRepuestos &datos)
    : wxDialog(wnd, -1, _T("wxListView")), data(datos)
{
    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);

    listView = new wxListView(this, idListCtrl);
    boxVertical->Add(listView, wxSizerFlags().Expand().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());
    SetSizerAndFit(boxVertical);

    InsertaColumnas(listView);
    AsignaImagenes(listView);
    InsertaItems(listView);
}

listview::~listview()
{
    //dtor
}

void listview::OnClose(wxCloseEvent& event) {
    Destroy();
}

void listview::InsertaColumnas(wxListView* listView) {
    listView->InsertColumn(0, _T("Nombre"));
    wxListItem listitem;
    listitem.SetAlign(wxLIST_FORMAT_RIGHT);
    listitem.SetText(_T("Stock"));
    listView->InsertColumn(1, listitem);
    listitem.SetText(_T("Consumido"));
    listView->InsertColumn(2, listitem);
}

void listview::AsignaImagenes(wxListView* listView) {
    wxBitmap bm;
    wxImageList* listaImagenGrande=new wxImageList(48, 48, true, 3);
    bm.LoadFile("tornillo-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    bm.LoadFile("tuerca-48.png", wxBITMAP_TYPE_PNG);
    listaImagenGrande->Add(bm);
    listView->AssignImageList(listaImagenGrande, wxIMAGE_LIST_NORMAL);
    wxImageList* listaImagenPequena=new wxImageList(16, 16, true, 3);
    bm.LoadFile("tornillo-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    bm.LoadFile("tuerca-16.png", wxBITMAP_TYPE_PNG);
    listaImagenPequena->Add(bm);
    listView->AssignImageList(listaImagenPequena, wxIMAGE_LIST_SMALL);
}

void listview::InsertaItems(wxListView* listView) {
    wxString cad;
    wxListItem item;
    item.Clear();
    for(size_t i=0; i<data.almacen.size(); i++) {
        item.SetId(i);
        item.SetColumn(0);
        item.SetImage(data.almacen[i].tipo);
        item.SetText(data.almacen[i].nombre);
        item.SetData(i);
        //item.SetMask(wxLIST_MASK_TEXT | wxLIST_MASK_IMAGE | wxLIST_MASK_DATA);
        listView->InsertItem(item);
        cad.Clear();
        cad << data.almacen[i].stock;
        listView->SetItem(i, 1, cad);
        cad.Clear();
        cad << data.almacen[i].consumido;
        listView->SetItem(i, 2, cad);
    }
    for(size_t i=0; i<data.marcado.size(); i++) {
        item.SetId(data.marcado[i]);
        item.SetState(wxLIST_STATE_SELECTED);
        item.SetMask(wxLIST_MASK_STATE);
        listView->SetItem(item);
    }
    listView->SetColumnWidth(0, wxLIST_AUTOSIZE);
}

void listview::OnOk(wxCommandEvent& event) {
    long item=-1;

    data.marcado.clear();
    //item = listView->GetFirstSelected();
    //if(item!=-1) data.marcado.push_back(item);
    do {
        item = listView->GetNextSelected(item);
        if(item!=-1) data.marcado.push_back(item);
    } while(item!=-1);
    EndModal(wxID_OK);
}
