/***************************************************************
 * Name:      wx018Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-03-23
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "wx018Main.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(wx018Frame, wxFrame)
    EVT_CLOSE(wx018Frame::OnClose)
    EVT_MENU(idMenuQuit, wx018Frame::OnQuit)
    EVT_MENU(idMenuAbout, wx018Frame::OnAbout)
    EVT_MENU(idMenuListCtrl1, wx018Frame::OnListCtrl1)
    EVT_MENU(idMenuListCtrl2, wx018Frame::OnListCtrl2)
    EVT_MENU(idMenuListCtrl3, wx018Frame::OnListCtrl3)
    EVT_MENU(idMenuListCtrl4, wx018Frame::OnListCtrl4)
    EVT_MENU(idMenuListCtrl5, wx018Frame::OnListCtrl5)
    EVT_MENU(idMenuListView, wx018Frame::OnListView)
END_EVENT_TABLE()

wx018Frame::wx018Frame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title)
{
    wxImage::AddHandler(new wxPNGHandler);
    datos.almacen.push_back({1, tornillo, _T("Tornillo M3"), 100, 324});
    datos.almacen.push_back({2, tornillo, _T("Tornillo M5"), 80, 58});
    datos.almacen.push_back({3, tuerca, _T("Tuerca M3"), 112, 124});
    datos.almacen.push_back({4, tuerca, _T("Tuerca M4"), 65, 952});
    datos.almacen.push_back({5, tuerca, _T("Tuerca M5"), 74, 12});

    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Salir de la aplicación"));
    mbar->Append(fileMenu, _T("&Fichero"));

    wxMenu* listMenu = new wxMenu(_T(""));
    listMenu->Append(idMenuListCtrl1, _T("&Selección de ítems"), _T("Ejemplo de control wxListCtrl"));
    listMenu->Append(idMenuListCtrl2, _T("&Marcas de chequeo y ocultar columnas"), _T("Ejemplo de control wxListCtrl con check"));
    listMenu->Append(idMenuListCtrl3, _T("&Ordenar ítems"), _T("Ejemplo de orden en control wxListCtrl"));
    listMenu->Append(idMenuListCtrl4, _T("&Editar ítems"), _T("Ejemplo de edición en control wxListCtrl"));
    listMenu->Append(idMenuListCtrl5, _T("wxListCtrl &virtual"), _T("Ejemplo de wxListCtrl virtual"));
    listMenu->Append(idMenuListView,  _T("&wxListViewl"), _T("Ejemplo de wxListView"));
    mbar->Append(listMenu, _T("&Controles de lista"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre esta aplicación"));
    mbar->Append(helpMenu, _T("&Ayuda"));

    SetMenuBar(mbar);

    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_T("Hola"),0);
    SetStatusText(wxbuildinfo(short_f), 1);
}

wx018Frame::~wx018Frame()
{
}

void wx018Frame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void wx018Frame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void wx018Frame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _T("Bienvenido a ..."));
}

void wx018Frame::OnListCtrl1(wxCommandEvent& event) {
    wxString cad;
    listctrl1 *dlg = new listctrl1(this, datos);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Selección: ");
        for(size_t i=0; i<datos.marcado.size(); i++) {
            cad << datos.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("ListCtrl1"));
    }
}

void wx018Frame::OnListCtrl2(wxCommandEvent& event) {
    wxString cad;
    listctrl2 *dlg = new listctrl2(this, datos);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Marcadas: ");
        for(size_t i=0; i<datos.marcado.size(); i++) {
            cad << datos.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("ListCtrl2"));
    }
}

void wx018Frame::OnListCtrl3(wxCommandEvent& event) {
    wxString cad;
    listctrl3 *dlg = new listctrl3(this, datos);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Selección: ");
        for(size_t i=0; i<datos.marcado.size(); i++) {
            cad << datos.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("ListCtrl3"));
    }
}

void wx018Frame::OnListCtrl4(wxCommandEvent& event) {
    wxString cad;
    listctrl4 *dlg = new listctrl4(this, datos);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Selección: ");
        for(size_t i=0; i<datos.marcado.size(); i++) {
            cad << datos.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("ListCtrl4"));
    }
}

void wx018Frame::OnListCtrl5(wxCommandEvent& event) {
    wxString cad;
    listctrl5 *dlg = new listctrl5(this, datosC);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Selección: ");
        for(size_t i=0; i<datosC.marcado.size(); i++) {
            cad << datosC.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("ListCtrl5"));
    }
}

void wx018Frame::OnListView(wxCommandEvent& event) {
    wxString cad;
    listview *dlg = new listview(this, datos);
    if(dlg->ShowModal() == wxID_OK) {
        cad << _T("Selección: ");
        for(size_t i=0; i<datos.marcado.size(); i++) {
            cad << datos.marcado[i] << " ";
        }
        wxMessageBox(cad, _T("listview"));
    }
}
